@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Apply arpeggiation patterns to groups of notes
 
-our $VERSION = '0.0201';
+our $VERSION = '0.0202';
 
 use Moo;
 use strictures 2;
@@ -23,7 +23,7 @@ my $DISPATCH = {
 
 has type => (
     is      => 'rw',
-    isa     => sub { die "$_[0] does not look like a named type" unless $_[0] =~ /^\w+$/ },
+    isa     => sub { die "$_[0] is not a known named type" unless exists $DISPATCH->{$_[0]} },
     default => sub { 'up' },
 );
 
@@ -82,12 +82,7 @@ sub arp {
 
 sub build_pattern {
     my ($self, $type, $notes) = @_;
-    my $dispatch = {
-      up     => [ 0 .. $#$notes ],
-      down   => [ reverse(0 .. $#$notes) ],
-      random => [ map { rand @$notes } @$notes ]
-    };
-    return $dispatch->{$type};
+    return $self->arp_type($type)->($notes);
 }
 
 
@@ -118,7 +113,7 @@ Music::MelodicDevice::Arpeggiation - Apply arpeggiation patterns to groups of no
 
 =head1 VERSION
 
-version 0.0201
+version 0.0202
 
 =head1 SYNOPSIS
 
@@ -126,8 +121,12 @@ version 0.0201
 
   my $arp = Music::MelodicDevice::Arpeggiation->new;
 
+  $arp->arp_type('my_type', sub { my ($notes); return [0,2,1] }); # set a new type
+
   my $arped = $arp->arp([60,64,67], 1, 'updown', 3);
 
+  my $pattern = $arp->build_pattern('my-type', [60,65,67,69]); # [0,2,1]
+
 =head1 DESCRIPTION
 
 C<Music::MelodicDevice::Arpeggiation> applies arpeggiation patterns to
@@ -223,6 +222,7 @@ Known types and their code-ref values are:
 
   up     => sub { my ($notes) = @_; return [ 0 .. $#$notes ] },
   down   => sub { my ($notes) = @_; return [ reverse(0 .. $#$notes) ] },
+  updown => sub { my ($notes) = @_; return [ 0 .. $#$notes, reverse(1 .. $#$notes - 1) ] },
   random => sub { my ($notes) = @_; return [ map { rand @$notes } @$notes ] },
 
 =head1 SEE ALSO
